package jp.co.kayo.android.localplayer.util;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Vector;

import org.cmc.music.metadata.MusicMetadata;
import org.cmc.music.metadata.MusicMetadataSet;
import org.cmc.music.myid3.ID3Tag;
import org.cmc.music.myid3.ID3v2DataMapping;
import org.cmc.music.myid3.MyID3;
import org.cmc.music.myid3.MyID3Listener;
import org.cmc.music.myid3.MyID3v1;
import org.cmc.music.myid3.MyID3v2Read;

public class MyID3Dat extends MyID3 {
    private static final int ID3v2_HEADER_LENGTH = 10;
    
    @Override
    public MusicMetadataSet read(File file, MyID3Listener listener)
            throws IOException
    {
        try
        {
            if (file == null || !file.exists())
                return null;

            if (!(file.getName().toLowerCase().endsWith(".mp3") || file.getName().toLowerCase().endsWith(".dat")))
                return null;
            
            ID3Tag id3v1 = readID3v1(listener, file);
            ID3Tag.V2 id3v2 = readID3v2(listener, file, id3v1 != null);

            MusicMetadataSet result = MusicMetadataSet.factoryMethod(id3v1,
                    id3v2, file.getName(), file.getParentFile().getName());

            return result;
        } catch (Error e)
        {

            throw e;
        } catch (IOException e)
        {

            throw e;
        }
    }
    
    
    private byte[] readID3v2Head(File file) throws IOException
    {

        if (file == null || !file.exists())
            return null;

        long length = file.length();

        if (length < ID3v2_HEADER_LENGTH)
            return null;

        InputStream is = null;
        try
        {
            is = new FileInputStream(file);
            is = new BufferedInputStream(is, 8192);

            byte header[];
            header = readArray(is, ID3v2_HEADER_LENGTH);

            if (header[0] != 0x49) // I
                return null;
            if (header[1] != 0x44) // D
                return null;
            if (header[2] != 0x33) // 3
                return null;

            int flags = header[5];
            boolean has_footer = (flags & (1 << 4)) > 0;

            Number tagLength = MyID3v2Read.readSynchsafeInt(header, 6);
            if (tagLength == null)
                return null;

            int bodyLength = tagLength.intValue();
            if (has_footer)
                bodyLength += ID3v2_HEADER_LENGTH;

            if (ID3v2_HEADER_LENGTH + bodyLength > length)
                return null;

            byte body[] = readArray(is, bodyLength);

            byte result[] = new byte[header.length + body.length];

            System.arraycopy(header, 0, result, 0, header.length);
            System.arraycopy(body, 0, result, header.length, body.length);

            return result;
        } finally
        {
            try
            {
                if (is != null)
                    is.close();
            } catch (IOException e)
            {

            }
        }
    }    
    private ID3Tag.V2 readID3v2(MyID3Listener listener, File file,
            boolean hasId3v1) throws IOException
    {
        if (file == null || !file.exists())
            return null;

        byte bytes[] = null;
        bytes = readID3v2Tail(file, hasId3v1);
        if (bytes == null)
            bytes = readID3v2Head(file);

        if (bytes == null)
            return null;

        if (null != listener)
            listener.log("ID3v2 tag found: " + bytes.length + " bytes");

        MyID3v2Read parser = new MyID3v2Read(listener,
                new ByteArrayInputStream(bytes), false);
        while (!parser.isComplete())
        {
            parser.iteration();
        }
        if (parser.isError())
        {
            if (listener != null)
                listener.log("id3v2 error", parser.getErrorMessage());
            
            parser.dump();
            return null;
        }

        if (!parser.hasTags())
            return null;

        Vector tags = parser.getTags();

        MusicMetadata values = new ID3v2DataMapping().process(tags);

        byte version_major = parser.getVersionMajor();
        byte version_minor = parser.getVersionMinor();

        if (null != listener)
            listener.log();

        return new ID3Tag.V2(version_major, version_minor, bytes, values, tags);
    }
    
    
    private byte[] readID3v2Tail(File file, boolean hasId3v1)
            throws IOException
    {
        if (file == null || !file.exists())
            return null;

        long length = file.length();

        int index = hasId3v1 ? 128 : 0;
        index += ID3v2_HEADER_LENGTH;

        if (index > length)
            return null;

        InputStream is = null;
        try
        {
            is = new FileInputStream(file);
            is = new BufferedInputStream(is, 8192);

            is.skip(length - index);

            byte footer[];
            footer = readArray(is, ID3v2_HEADER_LENGTH);

            if (footer[2] != 0x33) // 3
                return null;
            if (footer[1] != 0x44) // D
                return null;
            if (footer[0] != 0x49) // I
                return null;

            Number tagLength = MyID3v2Read.readSynchsafeInt(footer, 6);
            if (tagLength == null)
                return null;

            int bodyLength = tagLength.intValue();
            if (index + bodyLength > length)
                return null;

            is.close();
            is = null;

            is = new FileInputStream(file);
            is = new BufferedInputStream(is, 8192);

            long skip = length;
            skip -= ID3v2_HEADER_LENGTH;
            skip -= bodyLength;
            skip -= ID3v2_HEADER_LENGTH;
            if (hasId3v1)
                skip -= 128;
            is.skip(skip);

            byte header_and_body[] = readArray(is, ID3v2_HEADER_LENGTH
                    + bodyLength + ID3v2_HEADER_LENGTH);

            byte result[] = header_and_body;

            return result;
        } finally
        {
            try
            {
                if (is != null)
                    is.close();
            } catch (IOException e)
            {

            }
        }
    }    
    
    
    private ID3Tag readID3v1(MyID3Listener listener, File file)
            throws IOException
    {
        if (file == null || !file.exists())
            return null;

        long length = file.length();

        if (length < 128)
            return null;

        byte bytes[];
        InputStream is = null;
        try
        {
            is = new FileInputStream(file);
            is = new BufferedInputStream(is, 8192);

            is.skip(length - 128);

            bytes = readArray(is, 128);
        } finally
        {
            try
            {
                if (is != null)
                    is.close();
            } catch (IOException e)
            {

            }
        }

        if (bytes[0] != 'T')
            return null;
        if (bytes[1] != 'A')
            return null;
        if (bytes[2] != 'G')
            return null;

        if (null != listener)
            listener.log("ID3v1 tag found.");

        MyID3v1 id3v1 = new MyID3v1();
        MusicMetadata tags = id3v1.parseTags(listener, bytes);

        return new ID3Tag.V1(bytes, tags);
    }
    
    private final byte[] readArray(InputStream is, int length)
            throws IOException
    {
        byte result[] = new byte[length];
        int total = 0;
        while (total < length)
        {
            int read = is.read(result, total, length - total);
            if (read < 0)
                throw new IOException("bad read");
            total += read;
        }
        return result;
    }
}
